SystemOrganization addCategory: #'Discovery-Core'!
SystemOrganization addCategory: #'Discovery-Test'!

RbtClient subclass: #DsSearcher
	instanceVariableNames: 'responseHandler returnSemaphore entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsSearcher commentStamp: 'mu 4/2/2006 16:57' prior: 0!
I can search network services by broadcasting (multicasting) queries. While locator passively waits service availability events, I actively search services. I and locator can be used supplementally.!

!DsSearcher class methodsFor: 'private' stamp: 'mu 3/8/2006 22:24'!
bind
	^self bindForMulticast! !

!DsSearcher class methodsFor: 'private' stamp: 'mu 3/8/2006 22:24'!
bindForBroadcast
	^self bindForBroadcast: DsSettings defaultBroadcastPort + 1! !

!DsSearcher class methodsFor: 'private' stamp: 'mu 4/4/2006 23:46'!
bindForMulticast
	^self bindGroup: DsSettings defaultMulticastAddress port: DsSettings defaultMulticastPort + 1! !

!DsSearcher class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:41'!
default
	^ DsSettings multicastIsDefault
		ifTrue: [self defaultForMulticast]
		ifFalse: [self defaultForBroadcast]! !

!DsSearcher class methodsFor: 'instance creation' stamp: 'mu 3/8/2006 22:23'!
defaultForBroadcast
	^self bindForBroadcast! !

!DsSearcher class methodsFor: 'instance creation' stamp: 'mu 3/8/2006 22:23'!
defaultForMulticast
	^self bindForMulticast! !

!DsSearcher methodsFor: 'actions' stamp: 'mu 4/5/2006 09:58'!
basicBind: ipAddressOrHost port: portNumber by: optionSymbol 
	super basicBind: ipAddressOrHost port: portNumber by: optionSymbol. 
	self bindHandler
	! !

!DsSearcher methodsFor: 'actions' stamp: 'mu 3/9/2006 22:32'!
bindHandler
	self isValid ifTrue: [ self prepareResponseHandler: self socket localPort].
	! !

!DsSearcher methodsFor: 'initialize-release' stamp: 'mu 3/11/2006 18:55'!
initialize
	super initialize.
	responseHandler ifNotNilDo: [:handler | handler release].
	self resetSemaphore.
	returnSemaphore _ Semaphore new.! !

!DsSearcher methodsFor: 'testing' stamp: 'mu 3/5/2006 18:59'!
isValid
	^socket notNil and: [socket isValid]! !

!DsSearcher methodsFor: 'private' stamp: 'mu 3/9/2006 22:32'!
prepareResponseHandler: backPort 
	responseHandler
		ifNotNilDo: [:handler | handler release].
	responseHandler := (RbtServer startOn: backPort)
				handlerBlock: [:message | 
					DsUtil log: message.
					self searchReturn: message].
	^ responseHandler! !

!DsSearcher methodsFor: 'initialize-release' stamp: 'mu 4/5/2006 23:41'!
release
	self initialize.
	super release! !

!DsSearcher methodsFor: 'actions' stamp: 'mu 3/5/2006 18:56'!
releaseSocket
	self responseHandler ifNotNilDo: [:handler | handler release].
	super releaseSocket.
	! !

!DsSearcher methodsFor: 'initialize-release' stamp: 'mu 3/11/2006 17:28'!
resetSemaphore
	self returnSemaphore ifNil: [^self].
	[self returnSemaphore isEmpty]
				whileFalse: [self returnSemaphore signal].! !

!DsSearcher methodsFor: 'accessing' stamp: 'mu 3/5/2006 18:52'!
responseHandler
	^responseHandler! !

!DsSearcher methodsFor: 'accessing' stamp: 'mu 3/8/2006 21:39'!
returnSemaphore
	returnSemaphore ifNil: [returnSemaphore _ Semaphore new].
	^returnSemaphore! !

!DsSearcher methodsFor: 'actions-search' stamp: 'mu 3/8/2006 22:19'!
searchAll
	^self searchBy: DsSearchEntry all! !

!DsSearcher methodsFor: 'actions-search' stamp: 'mu 3/11/2006 17:29'!
searchBy: searchEntry 
	"Since this is UDP, you can not always get a correct answer."
	entries := nil.
	self sendData: searchEntry lineString.
	self returnSemaphore waitTimeoutSeconds: 1.
	entries
		ifNil: [self resetSemaphore.
			self bindHandler.
			entries := #()].
	^ entries! !

!DsSearcher methodsFor: 'callback' stamp: 'mu 4/5/2006 10:13'!
searchReturn: answer
	DsUtil log: answer.
	entries _ DsServiceEntry readLinesFrom: answer.
	self returnSemaphore signal.
	^entries! !

RbtServer subclass: #DsLocator
	instanceVariableNames: 'entryDict cleanUpProcess entriesProtect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

DsLocator class
	instanceVariableNames: 'locators'!

!DsLocator commentStamp: 'mu 4/2/2006 16:50' prior: 0!
I am a network service locator. I will receive availabilities of registered services. I will raise #ServiceAdded event when a new service is added to publisher. I will raise #ServiceRemoved when the service is no longer available.!

!DsLocator class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:40'!
default
	^ DsSettings multicastIsDefault
		ifTrue: [self defaultForMulticast]
		ifFalse: [self defaultForBroadcast]! !

!DsLocator class methodsFor: 'instance creation' stamp: 'mu 3/7/2006 20:24'!
defaultForBroadcast
	| inst |
	inst := self named: #defaultForBroadcast.
	inst startOn: DsSettings defaultBroadcastPort.
	^ inst! !

!DsLocator class methodsFor: 'instance creation' stamp: 'mu 4/4/2006 23:38'!
defaultForMulticast
	| inst |
	inst := self named: #defaultForMulticast.
	inst startOn: DsSettings defaultMulticastPort group: DsSettings defaultMulticastAddress.
	^inst! !

!DsLocator class methodsFor: 'class initialization' stamp: 'mu 3/12/2006 23:06'!
initialize
	"DsLocator initialize"
	super initialize.
	locators ifNotNilDo: [:col | col do:[:each | each release]].
	locators _ nil! !

!DsLocator class methodsFor: 'accessing' stamp: 'mu 3/7/2006 20:20'!
locators
	locators ifNil: [locators _ IdentityDictionary new].
	^locators! !

!DsLocator class methodsFor: 'instance creation' stamp: 'mu 3/7/2006 20:22'!
named: locatorName 
	| inst |
	inst := self locators
				at: locatorName
				ifAbsentPut: [self new].
	^ inst! !

!DsLocator methodsFor: 'private' stamp: 'mu 3/7/2006 19:58'!
basicStartOn: aPort by: optionSymbol
	super basicStartOn: aPort by: optionSymbol.
	self isRunning
		ifTrue: [self cleanUpProcess start].
	! !

!DsLocator methodsFor: 'builder' stamp: 'mu 3/5/2006 19:45'!
buildCleanUpProcess
	^ TdBlockTimer
		named: self className , ':' , #cleaner
		performBlock: [:timer | self cleanUp]
		intervalSeconds: self cleanUpIntervalSeconds! !

!DsLocator methodsFor: 'cleaning' stamp: 'mu 4/5/2006 22:14'!
cleanUp
	| expiredEntries |
	expiredEntries := self services
				select: [:each | each isExpired].
	expiredEntries
		do: [:each | entriesProtect
				critical: [self handleServiceRemoved: each]].
! !

!DsLocator methodsFor: 'defaults' stamp: 'mu 3/5/2006 19:45'!
cleanUpIntervalSeconds
	^10! !

!DsLocator methodsFor: 'accessing' stamp: 'mu 3/5/2006 19:42'!
cleanUpProcess
	cleanUpProcess
		ifNil: [cleanUpProcess := self buildCleanUpProcess].
	^ cleanUpProcess! !

!DsLocator methodsFor: 'accessing' stamp: 'mu 3/5/2006 18:35'!
entryDict
	entryDict ifNil: [entryDict _ IdentityDictionary new].
	^entryDict! !

!DsLocator methodsFor: 'receiving' stamp: 'mu 3/27/2006 12:42'!
handleReceived: entriesString 
	| entries |
	DsUtil log: entriesString.

	entries := [DsServiceEntry readLinesFrom: entriesString]
				on: Error
				do: [:ex | DsUtil log: ex description].

	entries
		do: [:each | entriesProtect
				critical: [each isByeBye
						ifTrue: [self handleServiceRemoved: each]
						ifFalse: [self handleServiceAdded: each ]]]! !

!DsLocator methodsFor: 'receiving' stamp: 'mu 3/12/2006 22:49'!
handleServiceAdded: serviceEntry 
	self entryDict at: serviceEntry id put: serviceEntry setLastUpdated.
	self triggerEvent: #ServiceAdded with: serviceEntry! !

!DsLocator methodsFor: 'receiving' stamp: 'mu 3/12/2006 22:50'!
handleServiceRemoved: serviceEntry 
	self entryDict
		removeKey: serviceEntry id
		ifAbsent: [^self].

	self triggerEvent: #ServiceRemoved with: serviceEntry! !

!DsLocator methodsFor: 'initialize-release' stamp: 'mu 4/5/2006 23:22'!
initialize
	super initialize.
	
	cleanUpProcess ifNotNilDo: [:p | p release].
	entryDict _ cleanUpProcess _ nil.

	"entriesProtect ifNotNilDo: [:p | p terminateProcess]."
	entriesProtect _ Semaphore forMutualExclusion! !

!DsLocator methodsFor: 'initialize-release' stamp: 'mu 4/5/2006 23:23'!
release
	self initialize.
	super release! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/7/2006 21:39'!
serviceNamed: serviceName
	| founds |
	founds _ (self servicesNamed: serviceName).
	founds ifEmpty: [^nil].
	^founds first! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/7/2006 21:32'!
services
	^self entryDict values! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/6/2006 08:12'!
servicesMatches: aBlock
	^self services select: aBlock! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/7/2006 21:30'!
servicesNamed: serviceName
	^self servicesMatches: [:each | each name = serviceName]! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/7/2006 21:31'!
servicesTyped: typeName
	^self servicesMatches: [:each | each type = typeName]! !

!DsLocator methodsFor: 'actions' stamp: 'mu 3/7/2006 21:39'!
serviceTyped: serviceName
	| founds |
	founds _ (self servicesTyped: serviceName).
	founds ifEmpty: [^nil].
	^founds first! !

!DsLocator methodsFor: 'running' stamp: 'mu 3/10/2006 10:08'!
start
	self cleanUpProcess notNil
		ifTrue: [| retry | 
			retry := 0.
			[self cleanUpProcess isStopping
				and: [retry < 10]]
				whileTrue: [DsUtil waitALittle.
					retry := retry + 1]].
	super start! !

!DsLocator methodsFor: 'running' stamp: 'mu 4/5/2006 23:15'!
stop
	super stop.
	cleanUpProcess ifNotNilDo: [:p | p stop]! !

RbtServer subclass: #DsResponder
	instanceVariableNames: 'publisher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsResponder commentStamp: 'mu 4/2/2006 19:43' prior: 0!
I return back an answer for searcher.!

!DsResponder methodsFor: 'private' stamp: 'mu 3/8/2006 21:34'!
anwserFrom: aCollectionOfSearchEntry 
	^ aCollectionOfSearchEntry
		inject: ''
		into: [:sum :each | sum , each lineString]! !

!DsResponder methodsFor: 'receiving' stamp: 'mu 4/5/2006 08:33'!
handleReceived: queryString
	
	DsUtil log: queryString.
	(queryString beginsWith: (DsSearchEntry serchAllHeader)) ifTrue: [self returnSearchAll]! !

!DsResponder methodsFor: 'testing' stamp: 'mu 4/6/2006 00:32'!
isRunning
	^ publisher
		ifNil: [super isRunning]
		ifNotNil: [super isRunning
				and: [publisher isRunning]]! !

!DsResponder methodsFor: 'private' stamp: 'mu 3/8/2006 22:51'!
prepareResponseReturner
	
	| backIp backPort |
	backIp _ self socket remoteAddress.
	backPort _ self socket remotePort.
	"{backIp. backPort} inspect."

	^self prepareResponseReturner: backIp port: backPort.
	! !

!DsResponder methodsFor: 'private' stamp: 'mu 3/8/2006 10:02'!
prepareResponseReturner: ipAddress port: portNumber
	| ipAddressStr client |
	ipAddressStr _ NetNameResolver stringFromAddress: ipAddress.
	client _ RbtClient bind: ipAddressStr port: portNumber.
	^client	! !

!DsResponder methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:24'!
publisher
	^publisher! !

!DsResponder methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:24'!
publisher: aPublisher
	publisher _ aPublisher! !

!DsResponder methodsFor: 'initialize-release' stamp: 'mu 3/11/2006 18:46'!
release
	super release.
	publisher _ nil! !

!DsResponder methodsFor: 'returning' stamp: 'mu 4/5/2006 10:13'!
returnSearchAll
	
	| returner |
	returner _ self prepareResponseReturner.
	returner sendData: (self anwserFrom: self publisher services).! !

AbInitializer subclass: #DsInitializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsInitializer class methodsFor: 'system startup' stamp: 'mu 4/2/2006 18:21'!
systemStartUp: resuming
	RbtClient localAddressGetter: DsUtil.! !

!TcpService methodsFor: '*Discovery-publishing' stamp: 'mu 4/2/2006 17:12'!
startPublish
	| serviceEntry |
	serviceEntry := DsServiceEntry
				type: self className
				version: ''
				name: self name
				host: DsUtil localHostName
				port: self portNumber.
	DsPublisher default
		registerService: serviceEntry
		while: [self isRunning]! !

!TcpService methodsFor: '*Discovery-publishing' stamp: 'mu 3/11/2006 18:24'!
stopPublish
	DsPublisher default
		removeServiceType: self className version: '' name: self name! !

TdTimer subclass: #DsServiceEntryPublisher
	instanceVariableNames: 'publisingPorts rbtClient serviceEntry isRunningBlock parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsServiceEntryPublisher commentStamp: 'mu 4/2/2006 19:08' prior: 0!
I publish a single service entry for parent (root) publisher.!

!DsServiceEntryPublisher class methodsFor: 'defaults' stamp: 'mu 3/5/2006 19:57'!
defaultPriority

	^Processor userBackgroundPriority! !

!DsServiceEntryPublisher class methodsFor: 'instance creation' stamp: 'mu 3/6/2006 08:26'!
new
	^self basicNew initialize! !

!DsServiceEntryPublisher class methodsFor: 'instance creation' stamp: 'mu 3/5/2006 20:25'!
on: serviceEntry
	^self new serviceEntry: serviceEntry! !

!DsServiceEntryPublisher methodsFor: 'actions-port' stamp: 'mu 3/11/2006 18:12'!
addPort: portNumber
	self publisingPorts add: portNumber! !

!DsServiceEntryPublisher methodsFor: 'binding' stamp: 'mu 3/7/2006 09:56'!
bindForBroadcast: portNumber 
	self publisingPorts ifEmpty: [self addPort: portNumber].
	self rbtClient bindForBroadcast: portNumber ! !

!DsServiceEntryPublisher methodsFor: 'binding' stamp: 'mu 4/5/2006 10:04'!
bindGroup: groupAddr port: portNumber 
	groupAddr ifNil: [^self bindForBroadcast: portNumber].

	self publisingPorts ifEmpty: [self addPort: portNumber].
	self rbtClient bindGroup: groupAddr port: portNumber ! !

!DsServiceEntryPublisher methodsFor: 'builder' stamp: 'mu 3/11/2006 17:25'!
buildMessage
	^self serviceEntry ifNotNilDo: [:ent | ent lineString ]! !

!DsServiceEntryPublisher methodsFor: 'actions' stamp: 'mu 3/11/2006 18:16'!
byebye
	self serviceEntry ifNotNilDo: [:ent | ent byebye].
	! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/11/2006 17:20'!
isRunningBlock
	^isRunningBlock! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/11/2006 17:20'!
isRunningBlock: aBlock
	isRunningBlock _ aBlock! !

!DsServiceEntryPublisher methodsFor: 'private' stamp: 'mu 3/5/2006 20:59'!
log: anException 
	DsUtil log: anException! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/11/2006 18:12'!
parent
	^parent! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/11/2006 18:12'!
parent: aPublisher
	parent _ aPublisher! !

!DsServiceEntryPublisher methodsFor: 'running' stamp: 'mu 3/5/2006 15:00'!
perform
	self publish! !

!DsServiceEntryPublisher methodsFor: 'running' stamp: 'mu 3/27/2006 12:24'!
publish
	self publisingPorts
		do: [:port | 
			self
				safelyDo: [self send: self buildMessage].
			DsUtil waitALittle].

	(self serviceEntry notNil and: [self serviceEntry isByeBye]) ifTrue: [self release].

	isRunningBlock notNil ifTrue: [(isRunningBlock value) ifFalse: [self remove; release]]! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/1/2006 23:56'!
publisingPort
	self publisingPorts size = 1 ifFalse: [^nil].
	^self publisingPorts first.! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:22'!
publisingPorts
	publisingPorts ifNil: [publisingPorts _ Set new].
	^publisingPorts! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:22'!
publisingPorts: aCollection
	publisingPorts _ aCollection! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/1/2006 23:55'!
publisingPort: portNumber
	self publisingPorts: {portNumber}! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 14:55'!
rbtClient
	rbtClient ifNil: [rbtClient _ RbtClient new].
	^rbtClient! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 14:55'!
rbtClient: aRbtClient
	rbtClient _ aRbtClient! !

!DsServiceEntryPublisher methodsFor: 'registration' stamp: 'mu 3/10/2006 20:42'!
register
	"never registered as ApplicationService"! !

!DsServiceEntryPublisher methodsFor: 'initialize-release' stamp: 'mu 3/11/2006 18:22'!
release
	super release.
	rbtClient ifNotNilDo: [:c | c release].
	
	publisingPorts _ rbtClient _ serviceEntry _ isRunningBlock _ parent _ nil! !

!DsServiceEntryPublisher methodsFor: 'actions' stamp: ''!
remove
	| ent |
	ent := self serviceEntry.
	ent	ifNil: [^ self].
	self parent
		ifNotNilDo: [:p | p
				removeServiceType: ent type
				version: ent version
				name: ent name]! !

!DsServiceEntryPublisher methodsFor: 'actions-port' stamp: 'mu 3/11/2006 18:13'!
removePort: portNumber
	self publisingPorts remove: portNumber ifAbsent:[]! !

!DsServiceEntryPublisher methodsFor: 'private' stamp: 'mu 3/1/2006 23:28'!
safelyDo: aBlock
	[aBlock value] on: Exception do: [:ex | self log: ex].! !

!DsServiceEntryPublisher methodsFor: 'sending' stamp: 'mu 3/10/2006 19:49'!
send: data 
	DsUtil log: data asString.
	self rbtClient sendData: data! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:24'!
serviceEntry
	^serviceEntry! !

!DsServiceEntryPublisher methodsFor: 'accessing' stamp: 'mu 3/6/2006 23:11'!
serviceEntry: aServiceEntry
	serviceEntry _ aServiceEntry.

	self intervalSeconds: aServiceEntry maxAge! !

AbUtil subclass: #DsUtil
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsUtil class methodsFor: 'actions' stamp: 'mu 3/7/2006 20:54'!
logError: anException 
	"DsUtil log: anException"
	DsSettings isDebugging
		ifTrue: [Transcript cr; show: thisContext sender sender asString , ': ' , anException description.
			anException return]! !

!DsUtil class methodsFor: 'actions' stamp: 'mu 3/7/2006 20:53'!
log: anExceptionOrString 
	"DsUtil log: 'hello'"
	(anExceptionOrString isKindOf: Exception)
		ifTrue: [^self logError: anExceptionOrString].

	DsSettings isDebugging
		ifTrue: [Transcript cr; show: thisContext sender asString , ' ' , anExceptionOrString]
		! !

!DsUtil class methodsFor: 'actions' stamp: 'mu 3/10/2006 10:04'!
waitALittle
	(Delay forMilliseconds: 100) wait! !

Object subclass: #DsPublisher
	instanceVariableNames: 'entryPublishers port groupAddress responder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

DsPublisher class
	instanceVariableNames: 'publishers'!

!DsPublisher commentStamp: 'mu 4/2/2006 16:47' prior: 0!
I am a network service publisher. I will broadcast (multicast) availabilities of registered services periodically.!

!DsPublisher class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:40'!
default
	^ DsSettings multicastIsDefault
		ifTrue: [self defaultForMulticast]
		ifFalse: [self defaultForBroadcast]! !

!DsPublisher class methodsFor: 'instance creation' stamp: 'mu 3/7/2006 21:37'!
defaultForBroadcast
	| inst |
	inst := self named: #defaultForBroadcast.
	inst port: DsSettings defaultBroadcastPort.
	inst start.
	^ inst! !

!DsPublisher class methodsFor: 'instance creation' stamp: 'mu 4/5/2006 10:05'!
defaultForMulticast
	| inst |
	inst := self named: #defaultForMulticast.
	inst groupAddress: DsSettings defaultMulticastAddress.
	inst port: DsSettings defaultMulticastPort.
	inst start.
	^ inst! !

!DsPublisher class methodsFor: 'class initialization' stamp: 'mu 3/12/2006 23:08'!
initialize
	"DsPublisher initialize"
	super initialize.
	publishers ifNotNilDo: [:col | col do:[:each | each release]].
	publishers _ nil! !

!DsPublisher class methodsFor: 'instance creation' stamp: 'mu 3/5/2006 20:21'!
named: publisherName 
	| inst |
	inst := self publishers
				at: publisherName
				ifAbsentPut: [self new].
	^ inst! !

!DsPublisher class methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:19'!
publishers
	publishers ifNil: [publishers _ IdentityDictionary new].
	^publishers! !

!DsPublisher methodsFor: 'private' stamp: 'mu 4/5/2006 10:03'!
basicRegisterService: serviceEntry 
	| oldEntryPublisher newEntryPublisher |
	(oldEntryPublisher := self entryPublishers
					detect: [:each | each serviceEntry = serviceEntry]
					ifNone: []) notNil
		ifTrue: [^ oldEntryPublisher serviceEntry: serviceEntry].

	newEntryPublisher := self serviceEntryPublisherClass on: serviceEntry.
	self isMulticast
		ifTrue: [newEntryPublisher bindGroup: self groupAddress port: self port]
		ifFalse: [newEntryPublisher bindForBroadcast: self port].
	newEntryPublisher parent: self.
	newEntryPublisher performThenStart.
	^ self entryPublishers add: newEntryPublisher! !

!DsPublisher methodsFor: 'private' stamp: 'mu 3/10/2006 19:50'!
basicRemoveService: serviceEntry
	| removedEntryPublishers |
	removedEntryPublishers _ self entryPublishers select: [:each | each serviceEntry = serviceEntry].
	removedEntryPublishers do: [:each | each byebye].
	self entryPublishers removeAllSuchThat: [:each | each serviceEntry = serviceEntry]! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:27'!
entryPublishers
	entryPublishers ifNil: [entryPublishers _ Set new].
	^entryPublishers! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 4/5/2006 10:03'!
groupAddress
	"nil means broadcast"
	^groupAddress! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 4/5/2006 10:03'!
groupAddress: addressString
	"nil means broadcast"
	^groupAddress _ addressString! !

!DsPublisher methodsFor: 'initialize-release' stamp: 'mu 4/6/2006 00:13'!
initialize
	responder ifNotNilDo: [:res | res release].
	responder _ nil.
	entryPublishers ifNotNilDo: [:pubs | pubs do: [:each | each release]].
	entryPublishers _ nil.
	port _ groupAddress _ nil.! !

!DsPublisher methodsFor: 'testing' stamp: 'mu 4/5/2006 10:04'!
isMulticast
	^self groupAddress notNil! !

!DsPublisher methodsFor: 'testing' stamp: 'mu 4/6/2006 00:38'!
isRunning
	entryPublishers ifNil: [^false].
	^entryPublishers anySatisfy: [:each | each isRunning]! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:50'!
port
	^port! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:50'!
port: anInteger
	port _ anInteger asString asInteger! !

!DsPublisher methodsFor: 'printing' stamp: 'mu 4/5/2006 23:01'!
printOn: aStream
	aStream nextPutAll: 'a ', self className, '['.
	entryPublishers ifNotNilDo: [:ps | aStream space; nextPutAll: ps asArray printString].
	port ifNotNilDo: [:p | aStream space; nextPutAll: p printString].
	groupAddress ifNotNilDo: [:a | aStream space; nextPutAll: a printString].
	aStream nextPutAll: ']'! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
registerServiceNamed: serviceName host: ipOrHostString port: portNumber maxAge: seconds 
	| service |
	service := DsServiceEntry
				name: serviceName
				host: ipOrHostString port: portNumber
				maxAge: seconds.
	^self basicRegisterService: service! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
registerServiceNamed: serviceName location: ipAndPortOrURL maxAge: seconds 
	| service |
	service := DsServiceEntry
				name: serviceName
				location: ipAndPortOrURL
				maxAge: seconds.
	^self basicRegisterService: service! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
registerServiceType: serviceType version: versionString name: serviceName host: ipOrHostString port: portNumber maxAge: seconds 
	| entry |
	entry := DsServiceEntry
				type: serviceType
				version: versionString
				name: serviceName
				host: ipOrHostString port: portNumber
				maxAge: seconds.
	^self basicRegisterService: entry! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
registerServiceType: serviceType version: versionString name: serviceName location: ipAndPOrtOrURL maxAge: seconds 
	| entry |
	entry := DsServiceEntry
				type: serviceType
				version: versionString
				name: serviceName
				location: ipAndPOrtOrURL
				maxAge: seconds.
	^self basicRegisterService: entry! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/7/2006 21:46'!
registerService: serviceEntry

	^ self basicRegisterService: (self serviceEntryFrom: serviceEntry)! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:25'!
registerService: serviceEntry while: runningBlock

	| entryPublisher |
	runningBlock value ifFalse: [^self].
	entryPublisher _ self basicRegisterService: (self serviceEntryFrom: serviceEntry).
	^entryPublisher isRunningBlock: runningBlock! !

!DsPublisher methodsFor: 'initialize-release' stamp: 'mu 3/11/2006 17:26'!
release
	self initialize! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
removeServiceNamed: serviceName
	| service |
	service := DsServiceEntry
				name: serviceName
				location: ''
				maxAge: 0.
	^self basicRemoveService: service

	! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/27/2006 12:42'!
removeServiceType: serviceType version: versionString name: serviceName
	| service |
	service := DsServiceEntry
				type: serviceType
				version: versionString
				name: serviceName
				location: ''
				maxAge: 0.
	^self basicRemoveService: service

	! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/7/2006 21:46'!
removeService: serviceEntry

	^ self basicRemoveService: (self serviceEntryFrom: serviceEntry)! !

!DsPublisher methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:22'!
responder
	responder ifNil: [self startResponder].
	^responder! !

!DsPublisher methodsFor: 'private' stamp: 'mu 3/27/2006 12:42'!
serviceEntryFrom: aServiceThing 
	| entry |
	(aServiceThing isMemberOf: DsServiceEntry)
		ifTrue: [entry := aServiceThing].
	aServiceThing isString
		ifTrue: [entry := DsServiceEntry readFrom: aServiceThing].
	(aServiceThing isMemberOf: Dictionary)
		ifTrue: [entry := DsServiceEntry fromDictionary: aServiceThing].
	^ entry! !

!DsPublisher methodsFor: 'factory' stamp: 'mu 3/5/2006 20:35'!
serviceEntryPublisherClass
	^DsServiceEntryPublisher
	! !

!DsPublisher methodsFor: 'accessing-services' stamp: 'mu 3/6/2006 21:45'!
services
	^self entryPublishers collect:[:each | each serviceEntry]! !

!DsPublisher methodsFor: 'running' stamp: 'mu 3/8/2006 08:22'!
start
	self entryPublishers do: [:each | each start].
	self startResponder! !

!DsPublisher methodsFor: 'private' stamp: 'mu 4/4/2006 23:38'!
startResponder
	responder ifNotNilDo: [:res | res release].
	responder := self isMulticast
				ifTrue: [DsResponder startOn: DsSettings defaultMulticastPort + 1 group: DsSettings defaultMulticastAddress]
				ifFalse: [DsResponder startOn: DsSettings defaultBroadcastPort + 1].
	responder publisher: self.
	^responder! !

!DsPublisher methodsFor: 'running' stamp: 'mu 3/8/2006 08:23'!
stop
	self entryPublishers do: [:each | each stop].
	self stopResponder! !

!DsPublisher methodsFor: 'private' stamp: 'mu 3/11/2006 18:49'!
stopResponder
	responder
		ifNotNilDo: [:res | res stop].
	! !

Object subclass: #DsSearchEntry
	instanceVariableNames: 'serviceEntry isAll'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsSearchEntry commentStamp: 'mu 4/2/2006 19:41' prior: 0!
I represent network service query.!

!DsSearchEntry class methodsFor: 'instance creation' stamp: 'mu 3/8/2006 08:00'!
all
	^self new! !

!DsSearchEntry class methodsFor: 'constants' stamp: 'mu 3/8/2006 19:48'!
serchAllHeader
	^'Search*'! !

!DsSearchEntry class methodsFor: 'constants' stamp: 'mu 3/8/2006 08:26'!
serchHeader
	^'Search'! !

!DsSearchEntry methodsFor: 'constants' stamp: 'mu 3/8/2006 08:27'!
header
	^ self isAll
		ifTrue: [self class serchAllHeader]
		ifFalse: [self class serchHeader]! !

!DsSearchEntry methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:05'!
isAll
	isAll ifNil: [isAll _ true].
	^isAll! !

!DsSearchEntry methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:04'!
isAll: aBoolean
	isAll _ aBoolean! !

!DsSearchEntry methodsFor: 'printing' stamp: 'mu 3/8/2006 08:06'!
lineString
	^String streamContents: [:str |
		str nextPutAll: self header; tab.
		self serviceEntry ifNotNilDo: [:ent | str nextPutAll: ent lineString].
		str cr.
	]! !

!DsSearchEntry methodsFor: 'printing' stamp: 'mu 3/8/2006 08:06'!
printOn: aStream 
	aStream nextPutAll: '['.
	aStream nextPutAll: self lineString.
	aStream skip: -1.
	aStream nextPutAll: ']'.! !

!DsSearchEntry methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:00'!
serviceEntry
	^serviceEntry! !

!DsSearchEntry methodsFor: 'accessing' stamp: 'mu 3/8/2006 08:00'!
serviceEntry: aServiceEntry
	serviceEntry _ aServiceEntry! !

!DsSearchEntry methodsFor: 'printing' stamp: 'mu 3/8/2006 08:07'!
storeString
	^self lineString! !

Object subclass: #DsServiceEntry
	instanceVariableNames: 'name type location version maxAge lastUpdatedTimestamp isByeBye'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

!DsServiceEntry commentStamp: 'mu 4/2/2006 16:58' prior: 0!
I represent network service description.!

!DsServiceEntry class methodsFor: 'examples' stamp: 'mu 3/27/2006 12:41'!
example1
	"DsServiceEntry example1"
	| inst str |
	inst _ self name: 'Service1' location: 'localshot:8823' maxAge: 200 .
	str _ inst lineString.

	(DsServiceEntry readFrom: str) inspect! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/5/2006 19:09'!
fromDictionary: aDictionary
	| inst |
	inst _ self new.
	aDictionary keysAndValuesDo: [:key :value |
		inst perform: (key, ':') asSymbol with: value
	].
	^inst! !

!DsServiceEntry class methodsFor: 'utilities' stamp: 'mu 3/11/2006 16:59'!
hostAndPortFromString: aString 
	"DsSeriveEntry ipAndPortFromString:'localhost:8080'"
	| tokens |
	tokens := aString findTokens: ':'.
	^ {tokens first. tokens last asNumber}! !

!DsServiceEntry class methodsFor: 'utilities' stamp: 'mu 3/11/2006 16:59'!
hostAndPortStringFrom: ipOrHostString port: portNumber
	^ipOrHostString asString, ':', portNumber asString! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:46'!
name: serviceName host: ipOrHostString port: portNumber 
	| inst |
	inst := self new.
	inst name: serviceName.
	inst location: (self hostAndPortStringFrom: ipOrHostString port: portNumber).
	inst maxAge: DsSettings defaultMaxAge.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 16:59'!
name: serviceName host: ipOrHostString port: portNumber maxAge: secs 
	| inst |
	inst := self new.
	inst name: serviceName.
	inst location: (self hostAndPortStringFrom: ipOrHostString port: portNumber).
	inst maxAge: secs.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:46'!
name: serviceName location: ipAndPOrtOrURL
	| inst |
	inst := self new.
	inst name: serviceName.
	inst location: ipAndPOrtOrURL.
	inst maxAge: DsSettings defaultMaxAge.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/5/2006 19:51'!
name: serviceName location: ipAndPOrtOrURL maxAge: secs 
	| inst |
	inst := self new.
	inst name: serviceName.
	inst location: ipAndPOrtOrURL.
	inst maxAge: secs.
	^ inst! !

!DsServiceEntry class methodsFor: 'reading' stamp: 'mu 3/7/2006 20:41'!
readFrom: aStreamOrString 
	^ self readLineFrom: aStreamOrString! !

!DsServiceEntry class methodsFor: 'reading' stamp: 'mu 3/10/2006 19:59'!
readLineFrom: aStreamOrString
	| str tabChar inst |
	aStreamOrString ifNil: [^nil].
	str := aStreamOrString isString
				ifTrue: [aStreamOrString readStream]
				ifFalse: [aStreamOrString].
	tabChar _ Character tab.
	inst _ self new.
	[str atEnd] whileFalse: [
		str peek = $!! 
			ifTrue: [inst byebye. 
					str next].
		inst type: (str upTo: tabChar).
		inst version: (str upTo: tabChar).
		inst name: (str upTo: tabChar).
		inst location: (str upTo: tabChar).
		inst maxAge: (str upTo: tabChar).
	].
	^inst

	! !

!DsServiceEntry class methodsFor: 'reading' stamp: 'mu 3/7/2006 20:43'!
readLinesFrom: linesStream
	| crChar str entries |
	crChar _ Character cr.
	entries _ OrderedCollection new.
	linesStream ifNil: [^#()].
	str := linesStream isString
				ifTrue: [linesStream readStream]
				ifFalse: [linesStream].
	[str atEnd] whileFalse: [ 
		entries add: (self readLineFrom: (str upTo: crChar)).
	].
	^entries! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:46'!
type: serviceType version: versionString name: serviceName host: ipOrHostString port: portNumber
	| inst |
	inst := self new.
	inst type: serviceType.
	inst version: versionString.
	inst name: serviceName.
	inst location: (self hostAndPortStringFrom: ipOrHostString port: portNumber).
	inst maxAge: DsSettings defaultMaxAge.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 16:59'!
type: serviceType version: versionString name: serviceName host: ipOrHostString port: portNumber maxAge: secs 
	| inst |
	inst := self new.
	inst type: serviceType.
	inst version: versionString.
	inst name: serviceName.
	inst location: (self hostAndPortStringFrom: ipOrHostString port: portNumber).
	inst maxAge: secs.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/11/2006 17:45'!
type: serviceType version: versionString name: serviceName location: ipAndPOrtOrURL
	| inst |
	inst := self new.
	inst type: serviceType.
	inst version: versionString.
	inst name: serviceName.
	inst location: ipAndPOrtOrURL.
	inst maxAge: DsSettings defaultMaxAge.
	^ inst! !

!DsServiceEntry class methodsFor: 'instance creation' stamp: 'mu 3/5/2006 19:51'!
type: serviceType version: versionString name: serviceName location: ipAndPOrtOrURL maxAge: secs 
	| inst |
	inst := self new.
	inst type: serviceType.
	inst version: versionString.
	inst name: serviceName.
	inst location: ipAndPOrtOrURL.
	inst maxAge: secs.
	^ inst! !

!DsServiceEntry methodsFor: 'converting' stamp: 'mu 3/5/2006 14:34'!
asDictionary
	| dict |
	dict _ Dictionary new.
	dict at: #name put: self name.
	dict at: #type put: self type.
	dict at: #location put: self location.
	dict at: #version put: self version.
	dict at: #maxAge put: self maxAge.
	^dict! !

!DsServiceEntry methodsFor: 'initialize-release' stamp: 'mu 3/10/2006 19:52'!
byebye
	isByeBye _ true! !

!DsServiceEntry methodsFor: 'comparing' stamp: 'mu 3/6/2006 21:47'!
hash
	^self id hash! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/12/2006 19:06'!
host
	^self hostAndPort first! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/11/2006 16:59'!
hostAndPort
	| url |
	^ ((url := self url) isString)
		ifTrue: [self class hostAndPortFromString: self location]
		ifFalse: [{url authority. url port}]! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/6/2006 08:31'!
id
	^(String streamContents: [:str |
		str nextPutAll: self type; tab.
		str nextPutAll: self version asString; tab.
		str nextPutAll: self name; tab.
	]) asSymbol! !

!DsServiceEntry methodsFor: 'initialize-release' stamp: 'mu 3/5/2006 19:51'!
initialize
	name _ ''.
	type _ #General.
	location _ ''.
	version _ '0'.
	maxAge _ 30! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/10/2006 19:54'!
isByeBye
	isByeBye ifNil: [isByeBye _ false].
	^isByeBye! !

!DsServiceEntry methodsFor: 'testing' stamp: 'mu 3/7/2006 20:57'!
isExpired
	^(Time millisecondClockValue - self lastUpdatedTimestamp) abs > ((self maxAge * 1000) * 2)! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/6/2006 07:58'!
lastUpdatedTimestamp
	lastUpdatedTimestamp ifNil: [self setLastUpdated].
	^lastUpdatedTimestamp! !

!DsServiceEntry methodsFor: 'printing' stamp: 'mu 3/10/2006 19:57'!
lineString
	^String streamContents: [:str |
		self isByeBye ifTrue: [str nextPut: $!!].
		str nextPutAll: self type; tab.
		str nextPutAll: self version asString; tab.
		str nextPutAll: self name; tab.
		str nextPutAll: self location; tab.
		str nextPutAll: self maxAge asString; cr.
	]! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
location

	^location! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
location: aValue

	location := aValue! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
maxAge

	^maxAge! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/2/2006 00:23'!
maxAge: aValue

	maxAge := aValue asString asInteger! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
name

	^name! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/2/2006 00:23'!
name: aValue

	name := aValue asString! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/12/2006 19:06'!
port
	^self hostAndPort last! !

!DsServiceEntry methodsFor: 'printing' stamp: 'mu 3/6/2006 21:48'!
printOn: aStream 
	aStream nextPutAll: '['.
	aStream nextPutAll: self lineString.
	aStream skip: -1.
	aStream nextPutAll: ']'.! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/6/2006 07:58'!
setLastUpdated
	lastUpdatedTimestamp _ Time millisecondClockValue! !

!DsServiceEntry methodsFor: 'printing' stamp: 'mu 3/2/2006 00:14'!
storeString
	^self lineString! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
type

	^type! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/2/2006 00:24'!
type: aValue

	type := aValue asString! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/11/2006 16:52'!
url
	| url |
	url _ self location asUrl.
	(url isMemberOf: GenericUrl) ifTrue: [^self location].
	^url! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/1/2006 22:40'!
version

	^version! !

!DsServiceEntry methodsFor: 'accessing' stamp: 'mu 3/2/2006 00:23'!
version: aValue

	version := aValue asString! !

!DsServiceEntry methodsFor: 'comparing' stamp: 'mu 3/12/2006 19:13'!
<= other
	self class ~= other class ifTrue: [^false].
	^self id <= other id and: [self location <= other location]! !

!DsServiceEntry methodsFor: 'comparing' stamp: 'mu 3/5/2006 18:32'!
= other
	self class ~= other class ifTrue: [^false].
	^self id = other id! !

TestCase subclass: #DsDiscoveryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Test'!

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 4/2/2006 16:25'!
testBroadcastDiscovery
	"self debug: #testBroadcastDiscovery"
	| loc pub |
	[loc := DsLocator defaultForBroadcast.
	self
		should: [(loc serviceNamed: 'superSwikiService1') isNil].
	pub := DsPublisher defaultForBroadcast.
	pub
		registerServiceNamed: 'superSwikiService1'
		host: 'localhost'
		port: 8080
		maxAge: 1.
	pub
		registerServiceNamed: 'otherSomeService1'
		host: 'localhost'
		port: 9090
		maxAge: 1.
	(Delay forSeconds: 3) wait.
	self
		shouldnt: [(loc servicesNamed: 'superSwikiService1') isNil].
	self
		shouldnt: [(loc servicesNamed: 'otherSomeService1') isNil].
	pub removeServiceNamed: 'superSwikiService1'.
	pub removeServiceNamed: 'otherSomeService1'.
	(Delay forSeconds: 3) wait.
	self
		should: [(loc serviceNamed: 'superSwikiService1') isNil].
	self
		should: [(loc serviceNamed: 'otherSomeService1') isNil]]
		ensure: [pub removeServiceNamed: 'superSwikiService1'.
			pub removeServiceNamed: 'otherSomeService1']! !

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 4/3/2006 23:18'!
testBroadcastSearch
	"self debug: #testBroadcastSearch"
	| pub searcher entries try |
	[pub := DsPublisher defaultForBroadcast.
	pub
		registerServiceNamed: 'superSwikiService1'
		host: 'localhost'
		port: 8080
		maxAge: 2.
	pub
		registerServiceNamed: 'otherSomeService1'
		host: 'localhost'
		port: 9090
		maxAge: 2.
	
	try _ 0.
	[try _ try + 1.
	(Delay forSeconds: 1) wait.
	searcher _ DsSearcher defaultForBroadcast.
	((entries _ searcher searchAll) isEmpty and: [try ~= 20])] whileTrue.

	self should: [entries includes: (DsServiceEntry name: 'superSwikiService1' host: 'localhost' port: 8080 maxAge: 2 )].
	self should: [entries includes: (DsServiceEntry name: 'otherSomeService1' host: 'localhost' port: 9090 maxAge: 2 )].
	
	pub removeServiceNamed: 'superSwikiService1'.
	pub removeServiceNamed: 'otherSomeService1'.

	entries _ searcher searchAll.
	self shouldnt: [entries includes: (DsServiceEntry name: 'superSwikiService1' host: 'localhost' port: 8080 maxAge: 2 )].
	self shouldnt: [entries includes: (DsServiceEntry name: 'otherSomeService1' host: 'localhost' port: 9090 maxAge: 2 )].

	] ensure: [searcher release.
			pub removeServiceNamed: 'superSwikiService1'.
			pub removeServiceNamed: 'otherSomeService1']! !

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 3/12/2006 23:13'!
testMulticastDiscovery
	"self debug: #testMulticastDiscovery"
	| loc pub |
	[loc := DsLocator defaultForMulticast.
	self
		should: [(loc serviceNamed: 'superSwikiService1') isNil].
	pub := DsPublisher defaultForMulticast.
	pub
		registerServiceNamed: 'superSwikiService1'
		host: 'localhost'
		port: 8080
		maxAge: 1.
	pub
		registerServiceNamed: 'otherSomeService1'
		host: 'localhost'
		port: 9090
		maxAge: 1.
	(Delay forSeconds: 3) wait.
	self
		shouldnt: [(loc servicesNamed: 'superSwikiService1') isNil].
	self
		shouldnt: [(loc servicesNamed: 'otherSomeService1') isNil].
	pub removeServiceNamed: 'superSwikiService1'.
	pub removeServiceNamed: 'otherSomeService1'.
	(Delay forSeconds: 3) wait.
	self
		should: [(loc serviceNamed: 'superSwikiService1') isNil].
	self
		should: [(loc serviceNamed: 'otherSomeService1') isNil]]
		ensure: [pub removeServiceNamed: 'superSwikiService1'.
			pub removeServiceNamed: 'otherSomeService1']! !

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 4/3/2006 23:17'!
testMulticastSearch
	"self debug: #testMulticastSearch"
	| pub searcher entries try |
	[pub := DsPublisher defaultForMulticast.
	pub
		registerServiceNamed: 'superSwikiService1'
		host: 'localhost'
		port: 8080
		maxAge: 2.
	pub
		registerServiceNamed: 'otherSomeService1'
		host: 'localhost'
		port: 9090
		maxAge: 2.
	
	try _ 0.
	[try _ try + 1.
	(Delay forSeconds: 1) wait.
	searcher _ DsSearcher defaultForMulticast.
	((entries _ searcher searchAll) isEmpty and: [try ~= 20])] whileTrue.

	self should: [entries includes: (DsServiceEntry name: 'superSwikiService1' host: 'localhost' port: 8080 maxAge: 2 )].
	self should: [entries includes: (DsServiceEntry name: 'otherSomeService1' host: 'localhost' port: 9090 maxAge: 2 )].
	
	pub removeServiceNamed: 'superSwikiService1'.
	pub removeServiceNamed: 'otherSomeService1'.

	entries _ searcher searchAll.
	self shouldnt: [entries includes: (DsServiceEntry name: 'superSwikiService1' host: 'localhost' port: 8080 maxAge: 2 )].
	self shouldnt: [entries includes: (DsServiceEntry name: 'otherSomeService1' host: 'localhost' port: 9090 maxAge: 2 )].

	] ensure: [searcher release.
			pub removeServiceNamed: 'superSwikiService1'.
			pub removeServiceNamed: 'otherSomeService1']! !

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 3/27/2006 12:42'!
testParseServiceEntries
	"self debug: #testParseServiceEntries"
	| serviceEnt1 serviceEnt2 serviceEnt3 |
	serviceEnt1 _ DsServiceEntry type: 'Test' version: 1 name: 'TestService' location: 'localhost' maxAge: 20.
	serviceEnt2 _  DsServiceEntry readLineFrom: serviceEnt1 lineString.
	
	self should: [serviceEnt1 = serviceEnt2].

	serviceEnt1 byebye.

	self should: [serviceEnt1 lineString beginsWith: ('!!', serviceEnt1 type)].

	serviceEnt3 _  DsServiceEntry readLineFrom: serviceEnt1 lineString.
	self should: [serviceEnt1 = serviceEnt3].

	self should: [serviceEnt3 isByeBye]

	
	 ! !

!DsDiscoveryTest methodsFor: 'testing' stamp: 'mu 3/7/2006 21:24'!
testServiceRegister
	"self debug: #testServiceRegister"
	| pub |
	pub _ DsPublisher new.
	pub port: DsSettings defaultBroadcastPort.
	
	pub registerServiceNamed: 'testService1' location: '192.168.1.1:8080' maxAge: 30 .
	pub registerServiceNamed: 'testService2' host: '192.168.1.1' port:8081 maxAge: 10 .

	self should: [pub services size = 2].

	pub removeServiceNamed: 'testService1'.
	pub registerServiceNamed: 'testService2' host: '192.168.1.1' port:8081 maxAge: 5 .

	self should: [pub services size = 1].

	self should: [pub services asArray first maxAge = 5].

	pub stop; initialize! !

AbSettings subclass: #DsSettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Discovery-Core'!

DsSettings class
	instanceVariableNames: 'isDebugging'!

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultBroadcastPort
	^self at: #defaultBroadcastPort ifAbsentPut:[9182]! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultBroadcastPort: port
	self at: #defaultBroadcastPort put: port! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultIntervalSeconds
	^self at: #defaultIntervalSeconds ifAbsentPut:[12]! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultIntervalSeconds: seconds
	self at: #defaultIntervalSeconds put: seconds! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMaxAge
	^self at: #defaultMaxAge ifAbsentPut:[20]! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMaxAge: integer
	
	self at: #defaultMaxAge put: integer! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMulticastAddress
	^self at: #defaultMulticastAddress ifAbsentPut:['239.255.255.123']! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMulticastAddress: addressString
	
	self at: #defaultMulticastAddress put: addressString! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMulticastPort
	^self at: #defaultMulticastPort ifAbsentPut:[9180]! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
defaultMulticastPort: port
	
	self at: #defaultMulticastPort put: port! !

!DsSettings class methodsFor: 'class initialization' stamp: 'mu 4/1/2006 21:20'!
initialize
	super initialize.
	isDebugging _ nil.
! !

!DsSettings class methodsFor: 'accessing' stamp: 'mu 3/5/2006 20:19'!
isDebugging
	isDebugging ifNil: [isDebugging _ false].
	^isDebugging! !

!DsSettings class methodsFor: 'accessing' stamp: 'mu 3/6/2006 23:08'!
isDebugging: aBoolean
	"DsSettings isDebugging: true"
	"DsSettings isDebugging: false"

	isDebugging _ aBoolean == true! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:20'!
multicastIsDefault
	"DsSettings multicastIsDefault"
	^self at: #multicastIsDefault ifAbsentPut:[true]! !

!DsSettings class methodsFor: 'defaults' stamp: 'mu 4/1/2006 21:21'!
multicastIsDefault: aBoolean
	
	self at: #multicastIsDefault put: aBoolean == true! !
DsLocator initialize!
DsPublisher initialize!
DsSettings initialize!
